@@ -25,14 +25,17 @@
 #include "refs.h"
 #include "index.h"
 #include "transports/smart.h"
+#include "transports/http.h"
 #include "streams/openssl.h"
 #include "streams/mbedtls.h"
 
-void git_libgit2_version(int *major, int *minor, int *rev)
+int git_libgit2_version(int *major, int *minor, int *rev)
 {
 	*major = LIBGIT2_VER_MAJOR;
 	*minor = LIBGIT2_VER_MINOR;
 	*rev = LIBGIT2_VER_REVISION;
+
+	return 0;
 }
 
 int git_libgit2_features(void)
@@ -57,6 +60,7 @@ int git_libgit2_features(void)
 extern size_t git_mwindow__window_size;
 extern size_t git_mwindow__mapped_limit;
 extern size_t git_indexer__max_objects;
+extern bool git_disable_pack_keep_file_checks;
 
 static int config_level_to_sysdir(int config_level)
 {
@@ -76,8 +80,8 @@ static int config_level_to_sysdir(int config_level)
 		val = GIT_SYSDIR_PROGRAMDATA;
 		break;
 	default:
-		giterr_set(
-			GITERR_INVALID, "invalid config path selector %d", config_level);
+		git_error_set(
+			GIT_ERROR_INVALID, "invalid config path selector %d", config_level);
 	}
 
 	return val;
@@ -193,7 +197,7 @@ int git_libgit2_opts(int key, ...)
 				error = git_mbedtls__set_cert_location(path, 1);
 		}
 #else
-		giterr_set(GITERR_SSL, "TLS backend doesn't support certificate locations");
+		git_error_set(GIT_ERROR_SSL, "TLS backend doesn't support certificate locations");
 		error = -1;
 #endif
 		break;
@@ -201,7 +205,7 @@ int git_libgit2_opts(int key, ...)
 		git__free(git__user_agent);
 		git__user_agent = git__strdup(va_arg(ap, const char *));
 		if (!git__user_agent) {
-			giterr_set_oom();
+			git_error_set_oom();
 			error = -1;
 		}
 
@@ -221,12 +225,12 @@ int git_libgit2_opts(int key, ...)
 			git__free(git__ssl_ciphers);
 			git__ssl_ciphers = git__strdup(va_arg(ap, const char *));
 			if (!git__ssl_ciphers) {
-				giterr_set_oom();
+				git_error_set_oom();
 				error = -1;
 			}
 		}
 #else
-		giterr_set(GITERR_SSL, "TLS backend doesn't support custom ciphers");
+		git_error_set(GIT_ERROR_SSL, "TLS backend doesn't support custom ciphers");
 		error = -1;
 #endif
 		break;
@@ -279,8 +283,16 @@ int git_libgit2_opts(int key, ...)
 		*(va_arg(ap, size_t *)) = git_indexer__max_objects;
 		break;
 
+	case GIT_OPT_DISABLE_PACK_KEEP_FILE_CHECKS:
+		git_disable_pack_keep_file_checks = (va_arg(ap, int) != 0);
+		break;
+
+	case GIT_OPT_ENABLE_HTTP_EXPECT_CONTINUE:
+		git_http__expect_continue = (va_arg(ap, int) != 0);
+		break;
+
 	default:
-		giterr_set(GITERR_INVALID, "invalid option key");
+		git_error_set(GIT_ERROR_INVALID, "invalid option key");
 		error = -1;
 	}
 
