@@ -34,7 +34,7 @@ int git_worktree_list(git_strarray *wts, git_repository *repo)
 	git_vector worktrees = GIT_VECTOR_INIT;
 	git_buf path = GIT_BUF_INIT;
 	char *worktree;
-	unsigned i, len;
+	size_t i, len;
 	int error;
 
 	assert(wts && repo);
@@ -136,11 +136,11 @@ static int open_worktree_dir(git_worktree **out, const char *parent, const char
 		goto out;
 	}
 
-	if ((wt->name = git__strdup(name)) == NULL
-	    || (wt->commondir_path = git_worktree__read_link(dir, "commondir")) == NULL
-	    || (wt->gitlink_path = git_worktree__read_link(dir, "gitdir")) == NULL
-	    || (parent && (wt->parent_path = git__strdup(parent)) == NULL)
-	    || (wt->worktree_path = git_path_dirname(wt->gitlink_path)) == NULL) {
+	if ((wt->name = git__strdup(name)) == NULL ||
+	    (wt->commondir_path = git_worktree__read_link(dir, "commondir")) == NULL ||
+	    (wt->gitlink_path = git_worktree__read_link(dir, "gitdir")) == NULL ||
+	    (parent && (wt->parent_path = git__strdup(parent)) == NULL) ||
+	    (wt->worktree_path = git_path_dirname(wt->gitlink_path)) == NULL) {
 		error = -1;
 		goto out;
 	}
@@ -149,7 +149,10 @@ static int open_worktree_dir(git_worktree **out, const char *parent, const char
 		goto out;
 	wt->gitdir_path = git_buf_detach(&gitdir);
 
-	wt->locked = !!git_worktree_is_locked(NULL, wt);
+	if ((error = git_worktree_is_locked(NULL, wt)) < 0)
+		goto out;
+	wt->locked = !!error;
+	error = 0;
 
 	*out = wt;
 
@@ -238,21 +241,21 @@ int git_worktree_validate(const git_worktree *wt)
 
 	if (!is_worktree_dir(wt->gitdir_path)) {
 		git_error_set(GIT_ERROR_WORKTREE,
-			"Worktree gitdir ('%s') is not valid",
+			"worktree gitdir ('%s') is not valid",
 			wt->gitlink_path);
 		return GIT_ERROR;
 	}
 
 	if (wt->parent_path && !git_path_exists(wt->parent_path)) {
 		git_error_set(GIT_ERROR_WORKTREE,
-			"Worktree parent directory ('%s') does not exist ",
+			"worktree parent directory ('%s') does not exist ",
 			wt->parent_path);
 		return GIT_ERROR;
 	}
 
 	if (!git_path_exists(wt->commondir_path)) {
 		git_error_set(GIT_ERROR_WORKTREE,
-			"Worktree common directory ('%s') does not exist ",
+			"worktree common directory ('%s') does not exist ",
 			wt->commondir_path);
 		return GIT_ERROR;
 	}
@@ -260,7 +263,7 @@ int git_worktree_validate(const git_worktree *wt)
 	return 0;
 }
 
-int git_worktree_add_init_options(git_worktree_add_options *opts,
+int git_worktree_add_options_init(git_worktree_add_options *opts,
 	unsigned int version)
 {
 	GIT_INIT_STRUCTURE_FROM_TEMPLATE(opts, version,
@@ -268,6 +271,12 @@ int git_worktree_add_init_options(git_worktree_add_options *opts,
 	return 0;
 }
 
+int git_worktree_add_init_options(git_worktree_add_options *opts,
+	unsigned int version)
+{
+	return git_worktree_add_options_init(opts, version);
+}
+
 int git_worktree_add(git_worktree **out, git_repository *repo,
 	const char *name, const char *worktree,
 	const git_worktree_add_options *opts)
@@ -397,20 +406,24 @@ out:
 int git_worktree_lock(git_worktree *wt, const char *reason)
 {
 	git_buf buf = GIT_BUF_INIT, path = GIT_BUF_INIT;
-	int err;
+	int error;
 
 	assert(wt);
 
-	if ((err = git_worktree_is_locked(NULL, wt)) < 0)
+	if ((error = git_worktree_is_locked(NULL, wt)) < 0)
 		goto out;
+	if (error) {
+		error = GIT_ELOCKED;
+		goto out;
+	}
 
-	if ((err = git_buf_joinpath(&path, wt->gitdir_path, "locked")) < 0)
+	if ((error = git_buf_joinpath(&path, wt->gitdir_path, "locked")) < 0)
 		goto out;
 
 	if (reason)
 		git_buf_attach_notowned(&buf, reason, strlen(reason));
 
-	if ((err = git_futils_writebuffer(&buf, path.ptr, O_CREAT|O_EXCL|O_WRONLY, 0644)) < 0)
+	if ((error = git_futils_writebuffer(&buf, path.ptr, O_CREAT|O_EXCL|O_WRONLY, 0644)) < 0)
 		goto out;
 
 	wt->locked = 1;
@@ -418,16 +431,19 @@ int git_worktree_lock(git_worktree *wt, const char *reason)
 out:
 	git_buf_dispose(&path);
 
-	return err;
+	return error;
 }
 
 int git_worktree_unlock(git_worktree *wt)
 {
 	git_buf path = GIT_BUF_INIT;
+	int error;
 
 	assert(wt);
 
-	if (!git_worktree_is_locked(NULL, wt))
+	if ((error = git_worktree_is_locked(NULL, wt)) < 0)
+		return error;
+	if (!error)
 		return 1;
 
 	if (git_buf_joinpath(&path, wt->gitdir_path, "locked") < 0)
@@ -448,22 +464,25 @@ int git_worktree_unlock(git_worktree *wt)
 int git_worktree_is_locked(git_buf *reason, const git_worktree *wt)
 {
 	git_buf path = GIT_BUF_INIT;
-	int ret;
+	int error, locked;
 
 	assert(wt);
 
 	if (reason)
 		git_buf_clear(reason);
 
-	if ((ret = git_buf_joinpath(&path, wt->gitdir_path, "locked")) < 0)
+	if ((error = git_buf_joinpath(&path, wt->gitdir_path, "locked")) < 0)
+		goto out;
+	locked = git_path_exists(path.ptr);
+	if (locked && reason &&
+	    (error = git_futils_readbuffer(reason, path.ptr)) < 0)
 		goto out;
-	if ((ret = git_path_exists(path.ptr)) && reason)
-		git_futils_readbuffer(reason, path.ptr);
 
+	error = locked;
 out:
 	git_buf_dispose(&path);
 
-	return ret;
+	return error;
 }
 
 const char *git_worktree_name(const git_worktree *wt)
@@ -478,7 +497,7 @@ const char *git_worktree_path(const git_worktree *wt)
 	return wt->worktree_path;
 }
 
-int git_worktree_prune_init_options(
+int git_worktree_prune_options_init(
 	git_worktree_prune_options *opts,
 	unsigned int version)
 {
@@ -487,10 +506,15 @@ int git_worktree_prune_init_options(
 	return 0;
 }
 
+int git_worktree_prune_init_options(git_worktree_prune_options *opts,
+	unsigned int version)
+{
+	return git_worktree_prune_options_init(opts, version);
+}
+
 int git_worktree_is_prunable(git_worktree *wt,
 	git_worktree_prune_options *opts)
 {
-	git_buf reason = GIT_BUF_INIT;
 	git_worktree_prune_options popts = GIT_WORKTREE_PRUNE_OPTIONS_INIT;
 
 	GIT_ERROR_CHECK_VERSION(
@@ -500,21 +524,25 @@ int git_worktree_is_prunable(git_worktree *wt,
 	if (opts)
 		memcpy(&popts, opts, sizeof(popts));
 
-	if ((popts.flags & GIT_WORKTREE_PRUNE_LOCKED) == 0 &&
-		git_worktree_is_locked(&reason, wt))
-	{
-		if (!reason.size)
-			git_buf_attach_notowned(&reason, "no reason given", 15);
-		git_error_set(GIT_ERROR_WORKTREE, "Not pruning locked working tree: '%s'", reason.ptr);
-		git_buf_dispose(&reason);
+	if ((popts.flags & GIT_WORKTREE_PRUNE_LOCKED) == 0) {
+		git_buf reason = GIT_BUF_INIT;
+		int error;
 
-		return 0;
+		if ((error = git_worktree_is_locked(&reason, wt)) < 0)
+			return error;
+
+		if (error) {
+			if (!reason.size)
+				git_buf_attach_notowned(&reason, "no reason given", 15);
+			git_error_set(GIT_ERROR_WORKTREE, "not pruning locked working tree: '%s'", reason.ptr);
+			git_buf_dispose(&reason);
+			return 0;
+		}
 	}
 
 	if ((popts.flags & GIT_WORKTREE_PRUNE_VALID) == 0 &&
-		git_worktree_validate(wt) == 0)
-	{
-		git_error_set(GIT_ERROR_WORKTREE, "Not pruning valid working tree");
+	    git_worktree_validate(wt) == 0) {
+		git_error_set(GIT_ERROR_WORKTREE, "not pruning valid working tree");
 		return 0;
 	}
 
@@ -546,7 +574,7 @@ int git_worktree_prune(git_worktree *wt,
 		goto out;
 	if (!git_path_exists(path.ptr))
 	{
-		git_error_set(GIT_ERROR_WORKTREE, "Worktree gitdir '%s' does not exist", path.ptr);
+		git_error_set(GIT_ERROR_WORKTREE, "worktree gitdir '%s' does not exist", path.ptr);
 		err = -1;
 		goto out;
 	}
@@ -566,7 +594,7 @@ int git_worktree_prune(git_worktree *wt,
 	git_buf_attach(&path, wtpath, 0);
 	if (!git_path_exists(path.ptr))
 	{
-		git_error_set(GIT_ERROR_WORKTREE, "Working tree '%s' does not exist", path.ptr);
+		git_error_set(GIT_ERROR_WORKTREE, "working tree '%s' does not exist", path.ptr);
 		err = -1;
 		goto out;
 	}
