@@ -1,5 +1,5 @@
 package Git::Raw::Remote;
-$Git::Raw::Remote::VERSION = '0.85';
+$Git::Raw::Remote::VERSION = '0.86';
 use strict;
 use warnings;
 
@@ -11,7 +11,7 @@ Git::Raw::Remote - Git remote class
 
 =head1 VERSION
 
-version 0.85
+version 0.86
 
 =head1 SYNOPSIS
 
@@ -144,11 +144,13 @@ The local OID of the reference (optional).
 
 =back
 
-=head2 fetch( [ \%fetch_opts ] )
+=head2 fetch( [ \%fetch_opts, [ \@refspecs ] ] )
 
 Download new data and update tips. Convenience function to connect to a remote,
-download the data, disconnect and update the remote-tracking branches. Valid fields for
-the C<%fetch_opts> hash are:
+download the data, disconnect and update the remote-tracking branches.
+C<@refspecs> is an optional list of refspecs to use for the negotiation with
+the server to determine the missing objects that need to be downloaded.  Valid
+fields for the C<%fetch_opts> hash are:
 
 =over 4
 
@@ -199,10 +201,10 @@ Connect to the remote. The C<$direction> should either be C<"fetch"> or C<"push"
 
 Disconnect the remote.
 
-=head2 download( [ \%fetch_opts ] )
+=head2 download( [ \%fetch_opts, [ \@refspecs ] ] )
 
 Download the remote packfile. See C<Git::Raw::Remote-E<gt>fetch()> for valid
-C<%fetch_opts> values.
+C<%fetch_opts> and C<@refspecs> values.
 
 =head2 upload( \@refspecs, [ \%push_opts ] )
 
